searchData={"items":[{"type":"module","title":"Dragonfly","doc":"Dragonfly remotely executes your application code on ephemeral nodes.\n\nDragonfly allows you to scale your application operations on a granular\nlevel **without rewriting your code**. For example, imagine the following function\nin your application that transcodes a video, saves the result to video storage,\nand updates the database:\n\n    def resize_video_quality(%Video{} = vid) do\n      path = \"#{vid.id}_720p.mp4\"\n      System.cmd(\"ffmpeg\", ~w(-i #{vid.url} -s 720x480 -c:a copy #{path}))\n      VideoStore.put_file!(\"videos/#{path}\", path)\n      {1, _} = Repo.update_all(from v in Video, where v.id == ^vid.id, set: [file_720p: path])\n      {:ok, path}\n    end\n\nThis works great locally and in production under no load, but video transcoding\nis necessarily an expensive CPU bound operation. In production, only a\nfew concurrent users can saturate your CPU and cause your entire application,\nweb requests, etc, to come to crawl. This is where folks typically reach for\nFaaS or external service solutions, but Dragonfly gives you a better way.\n\nSimply wrap your your existing code in a Dragonfly function and it will be executed\non a newly spawned, ephemeral node. Using Elixir and Erlang's built in distribution\nfeatures, entire function closures, including any state they close over, can be sent\nand executed on a remote node:\n\n    def resize_video_quality(%Video{} = video) do\n      Dragonfly.call(fn ->\n        path = \"#{vid.id}_720p.mp4\"\n        System.cmd(\"ffmpeg\", ~w(-i #{vid.url} -s 720x480 -c:a copy #{path}))\n        VideoStore.put_file!(\"videos/#{path}\", path)\n        {1, _} = Repo.update_all(from v in Video, where v.id == ^vid.id, set: [file_720p: path])\n        {:ok, path}\n      end)\n    end\n\nThat's it! The `%Video{}` struct in this example is captured inside the function\nand everything executes on the remotely spawned node, returning the result back to the\nparent node when it completes. Repo calls Just Work because the new node booted\nyour entire application, including the database Repo. As soon as the function is done\nexecuting, the ephemeral node is terminated. This means you can elastically scale\nyour app as load increases, and only pay for the resources you need at the time.","ref":"Dragonfly.html"},{"type":"module","title":"Backends - Dragonfly","doc":"The `Dragonfly.Backend` behavior defines an interface for spawning remote\napplication nodes and sending functions to them. By default, the\n`Dragonfly.LocalBackend` is used, which is great for development and test\nenvironments, as you can have your code simply execute locally in most cases\nand worry about scaling the operation only in production.\n\nFor production, Dragonfly provides the `Dragonfly.FlyBackend`, which uses\n(Fly.io)[https://fly.io]. Because Fly deploys a containerized machine of\nyour application, a single Fly API call can boot a machine running your\nexact Docker deployment image, allowing closures to be executed across\ndistributed nodes.\n\nDefault backends can be configured in your `config/runtime.exs`:\n\n    if config_env() == :prod do\n      config :dragonfly, :backend, Dragonfly.FlyBackend\n      config :dragonfly, Dragonfly.FlyBackend, token: System.fetch_env!(\"FLY_API_TOKEN\")\n      ...\n    end\n\nAnd then started in your supervision tree:\n\n    children = [\n      ...,\n      Dragonfly.FlyBackend,\n    ]","ref":"Dragonfly.html#module-backends"},{"type":"module","title":"Runners - Dragonfly","doc":"In practice, users will utilize the `Dragonfly.call/3` and `Dragonfly.cast/3` functions\nto accomplish most of their work. These functions are backed by a `Dragonfly.Runner`,\na lower-level primitive for executing functions on remote nodes.\n\nA `Dragonfly.Runner` is responsible for booting a new node, and executing concurrent\nfunctions on it. For example:\n\n    {:ok, runner} = Runner.start_link(backend: Dragonfly.FlyBackend)\n    :ok = Runner.remote_boot(runner)\n    Runner.call(runner, fn -> :operation1 end)\n    Runner.cast(runner, fn -> :operation2 end)\n    Runner.shutdown(runner)\n\nWhen a caller exits or crashes, the remote node will automatically be terminated.\nFor distributed erlang backends, like `Dragonfly.FlyBackend`, this will be\naccomplished automatically by the `Dragonfly.Terminator`, but other methods\nare possible.","ref":"Dragonfly.html#module-runners"},{"type":"module","title":"Pools - Dragonfly","doc":"Most workflows don't necessary need an entire node dedicated to a single function\nexecution. `Dragonfly.Pool` provides a higher-level abstraction that manages a\npool of runners. It provides elastic runner scaling, allowing a minimum and\nmaximum number of runners to be configured, and idle'd down as load decreases.\n\nPools give you elastic scale that maximizes the newly spawned hardware.\nAt the same time, you also want to avoid spawning unbound resources. You also\nwant to keep spawned nodes alive for a period of time to avoid the overhead\nof booting new ones before idleing them down. The following pool configuration\ntakes care of all of this for you:\n\n    children = [\n      ...,\n      Dragonfly.FlyBackend,\n      {Dragonfly.Pool,\n       name: App.FFMpegRunner,\n       min: 0,\n       max: 10,\n       max_concurrency: 5,\n       idle_shutdown_after: 60_000,\n    ]\n\nHere we add a `Dragonfly.Pool` to our application supervision tree, configuring\na minimum of 0 and maximum of 10 runners. This acheives \"scale to zero\" behavior\nwhile also allowing the pool to scale up to 10 runners when load increases.\nEach runner in the case will be able to execute up to 5 concurrent functions.\nThe runners will shutdown atter 60s of inactivity.\n\nCalling a pool is as simple as passing its name to the Dragonfly functions:\n\n    Dragonfly.call(App.FFMpegRunner, fn -> :operation1 end)","ref":"Dragonfly.html#module-pools"},{"type":"function","title":"Dragonfly.call/2","doc":"","ref":"Dragonfly.html#call/2"},{"type":"function","title":"Dragonfly.call/3","doc":"Calls a function in a remote runner.\n\nIf no runner is provided, a new one is linked to the caller and\nremotely booted.","ref":"Dragonfly.html#call/3"},{"type":"function","title":"Options - Dragonfly.call/3","doc":"* `:single_use` - if `true`, the runner will be terminated after the call. Defaults `false`.\n  * `:backend` - The backend to use. Defaults to `Dragonfly.LocalBackend`.\n  * `:log` - The log level to use for verbose logging. Defaults to `false`.\n  * `:single_use` -\n  * `:timeout` -\n  * `:connect_timeout` -\n  * `:shutdown_timeout` -\n  * `:task_su` -","ref":"Dragonfly.html#call/3-options"},{"type":"function","title":"Examples - Dragonfly.call/3","doc":"def my_expensive_thing(arg) do\n    Dragonfly.call(, fn ->\n      # i'm now doing expensive work inside a new node\n      # pubsub and repo access all just work\n      Phoenix.PubSub.broadcast(MyApp.PubSub, \"topic\", result)\n\n      # can return awaitable results back to caller\n      result\n    end)\n\nWhen the caller exits, the remote runner will be terminated.","ref":"Dragonfly.html#call/3-examples"},{"type":"behaviour","title":"Dragonfly.Backend","doc":"","ref":"Dragonfly.Backend.html"},{"type":"function","title":"Dragonfly.Backend.handle_info/2","doc":"","ref":"Dragonfly.Backend.html#handle_info/2"},{"type":"callback","title":"Dragonfly.Backend.handle_info/2","doc":"","ref":"Dragonfly.Backend.html#c:handle_info/2"},{"type":"function","title":"Dragonfly.Backend.impl/0","doc":"","ref":"Dragonfly.Backend.html#impl/0"},{"type":"function","title":"Dragonfly.Backend.init/1","doc":"","ref":"Dragonfly.Backend.html#init/1"},{"type":"callback","title":"Dragonfly.Backend.init/1","doc":"","ref":"Dragonfly.Backend.html#c:init/1"},{"type":"function","title":"Dragonfly.Backend.remote_boot/1","doc":"","ref":"Dragonfly.Backend.html#remote_boot/1"},{"type":"callback","title":"Dragonfly.Backend.remote_boot/1","doc":"","ref":"Dragonfly.Backend.html#c:remote_boot/1"},{"type":"function","title":"Dragonfly.Backend.remote_spawn_monitor/2","doc":"","ref":"Dragonfly.Backend.html#remote_spawn_monitor/2"},{"type":"callback","title":"Dragonfly.Backend.remote_spawn_monitor/2","doc":"","ref":"Dragonfly.Backend.html#c:remote_spawn_monitor/2"},{"type":"function","title":"Dragonfly.Backend.system_shutdown/0","doc":"","ref":"Dragonfly.Backend.html#system_shutdown/0"},{"type":"callback","title":"Dragonfly.Backend.system_shutdown/0","doc":"","ref":"Dragonfly.Backend.html#c:system_shutdown/0"},{"type":"module","title":"Dragonfly.FlyBackend","doc":"","ref":"Dragonfly.FlyBackend.html"},{"type":"function","title":"Dragonfly.FlyBackend.with_elapsed_ms/1","doc":"","ref":"Dragonfly.FlyBackend.html#with_elapsed_ms/1"},{"type":"module","title":"Dragonfly.Parent","doc":"TODO","ref":"Dragonfly.Parent.html"},{"type":"function","title":"Dragonfly.Parent.encode/3","doc":"TODO","ref":"Dragonfly.Parent.html#encode/3"},{"type":"function","title":"Dragonfly.Parent.get/0","doc":"TODO","ref":"Dragonfly.Parent.html#get/0"},{"type":"function","title":"Dragonfly.Parent.new/3","doc":"TODO","ref":"Dragonfly.Parent.html#new/3"},{"type":"module","title":"Dragonfly.Pool","doc":"Manages a pool of `Dragonfly.Runner`'s.\n\nPools support elastic growth and shrinking of the number of runners.","ref":"Dragonfly.Pool.html"},{"type":"module","title":"Examples - Dragonfly.Pool","doc":"children = [\n      ...,\n      {Dragonfly.Pool, name: MyRunner, min: 1, max: 10, max_concurrency: 100}\n    ]\n\n# TODO spin down after inactive_shutdown","ref":"Dragonfly.Pool.html#module-examples"},{"type":"function","title":"Dragonfly.Pool.call/3","doc":"TODO","ref":"Dragonfly.Pool.html#call/3"},{"type":"function","title":"Dragonfly.Pool.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Dragonfly.Pool.html#child_spec/1"},{"type":"function","title":"Dragonfly.Pool.start_link/1","doc":"TODO","ref":"Dragonfly.Pool.html#start_link/1"},{"type":"module","title":"Dragonfly.Pool.RunnerState","doc":"","ref":"Dragonfly.Pool.RunnerState.html"},{"type":"module","title":"Dragonfly.Pool.Supervisor","doc":"","ref":"Dragonfly.Pool.Supervisor.html"},{"type":"function","title":"Dragonfly.Pool.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Dragonfly.Pool.Supervisor.html#child_spec/1"},{"type":"function","title":"Dragonfly.Pool.Supervisor.init/1","doc":"","ref":"Dragonfly.Pool.Supervisor.html#init/1"},{"type":"function","title":"Dragonfly.Pool.Supervisor.start_link/1","doc":"","ref":"Dragonfly.Pool.Supervisor.html#start_link/1"},{"type":"module","title":"Dragonfly.Pool.WaitingState","doc":"","ref":"Dragonfly.Pool.WaitingState.html"},{"type":"module","title":"Dragonfly.Terminator.Caller","doc":"","ref":"Dragonfly.Terminator.Caller.html"}],"content_type":"text/markdown"}